@@ -1,7 +1,8 @@
 from __future__ import annotations
 
+import itertools
 from functools import reduce
-from typing import Collection, List, Callable, Union, Iterator, Iterable
+from typing import Collection, List, Callable, Union, Iterator, Iterable, Optional, Any
 from tqdm.auto import tqdm
 
 from .abstract_seq import AbstractSeq
@@ -9,7 +10,11 @@ from ...type import T, R
 from ....backend.parallel import ExecutorPool, ParallelType
 
 
-class Seq(AbstractSeq, Collection[T]):
+def _identity(x: T) -> T:
+    return x
+
+
+class Seq(AbstractSeq[T], Collection[T]):
 
     def __init__(self, items: Iterable[T]):
         if isinstance(items, Seq):
@@ -45,33 +50,39 @@ class Seq(AbstractSeq, Collection[T]):
     def __contains__(self, __x: object) -> bool:
         return self._items.__contains__(__x)
 
-    def map(self, f: Callable[[T], R], progress_bar: bool = False) -> Seq[R]:
-        items = self._items if not progress_bar else tqdm(self._items)
-        return Seq(map(f, items))
-
-    def for_each(self, f: Callable[[T], None], progress_bar: bool = False) -> None:
-        items = self._items if not progress_bar else tqdm(self._items)
-        for item in items:
-            f(item)
-
-    def parallel_map(self, f: Callable[[T], R], progress_bar: bool = False,
-        parallel_type: ParallelType = ParallelType.PROCESS
+    def map(self, f: Callable[[T], R], progress_bar: bool = False, parallel_type: Optional[ParallelType] = None,
+        **tqdm_args
     ) -> Seq[R]:
-        futures = []
+        if parallel_type is None:
+            items = self._items if not progress_bar else tqdm(self._items, **tqdm_args)
+            return Seq(map(f, items))
+        else:
+            futures = []
 
-        if f.__name__ == "<lambda>":
-            raise NotImplementedError("lambda function is not supported yet")
+            if f.__name__ == "<lambda>":
+                raise NotImplementedError("lambda function is not supported yet")
 
-        for item in self._items:
-            futures.append(ExecutorPool.submit(f, item, parallel_type=parallel_type))
+            for item in self._items:
+                futures.append(ExecutorPool.submit(f, item, parallel_type=parallel_type))
 
-        futures = tqdm(futures) if progress_bar else futures
-        return Seq(map(lambda future: future.result(), futures))
+            futures = tqdm(futures, **tqdm_args) if progress_bar else futures
+            return Seq(map(lambda future: future.result(), futures))
 
-    def parallel_for_each(self, f: Callable[[T], None], progress_bar: bool = False,
-        parallel_type: ParallelType = ParallelType.PROCESS
+    def for_each(self, f: Callable[[T], None], progress_bar: bool = False, parallel_type: Optional[ParallelType] = None,
+        **tqdm_args
     ) -> None:
-        self.parallel_map(f, progress_bar, parallel_type)
+        if parallel_type is None:
+            items = self._items if not progress_bar else tqdm(self._items, **tqdm_args)
+            for item in items:
+                f(item)
+        else:
+            self.map(f, progress_bar, parallel_type, **tqdm_args)
+
+    def with_for_each(self, f: Callable[[T], None], progress_bar: bool = False,
+        parallel_type: Optional[ParallelType] = None, **tqdm_args
+    ) -> Seq[T]:
+        self.for_each(f, progress_bar, parallel_type, **tqdm_args)
+        return self
 
     def filter(self, f: Callable[[T], bool]) -> Seq[T]:
         return Seq(filter(f, self._items))
@@ -91,12 +102,27 @@ class Seq(AbstractSeq, Collection[T]):
     def flat_map(self, f: Callable[[T], AbstractSeq[R]], progress_bar: bool = False) -> Seq[R]:
         return self.map(f, progress_bar).flatten()
 
+    def sort(self, key: Callable[[T], Any] = _identity, reverse: bool = False) -> Seq[T]:
+        return Seq(sorted(self._items, key=key, reverse=reverse))
+
     def take(self, n: int) -> Seq[T]:
         return self[:n]
 
     def skip(self, n: int) -> Seq[T]:
         return self[n:]
 
+    @property
+    def head(self) -> T:
+        return self._items[0]
+
+    @property
+    def tail(self) -> Seq[T]:
+        return self[1:]
+
+    def repeat(self, n: int) -> Seq[T]:
+        self._items = [item for item in itertools.repeat(self._items, n)]
+        return self
+
     def to_list(self) -> List[T]:
         return self._items
 
